#include<iostream>
/*#include <string>*/
#include <string.h>
#include<cstdlib>
#include<fstream>
#include<cmath>
#include<vector>
#include "Dal.h"
#include"errstr.h"
#include<stdlib.h>
/***************************************************************
*
*
*
* 
****************************************************************/

void loadData( string& fileName, vector<double>& ra, vector<double>& dec, 
    vector<float>& error, vector<float>& bmag, vector<float>& rmag );
bool createCatalogue(const string&  newSrcFile, const vector<double>& ra, 
		     const vector<double>& dec, const vector<float>& error, 
		     const vector<float>& bmag, const vector<float>& rmag  );

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	       vector<real32> data);

bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
	       vector<real64> data);


extern "C" void getsaspath_(char* pathstring)
{

    errstr << "Getting the SAS path for omatt"
           << Sas::message(Msg::App, Msg::Sparse);
 
    char* saspath=getenv("SAS_PATH");

    sprintf(pathstring,"%s",saspath);

}



extern "C" void getusno_(char* inputfilename, char* usnocatname, double &ra, double &dec, float &radius)
{
 
  char command[200];
  errstr << "Generating a subset of the USNO catalog for the image " << inputfilename 
           << Sas::message(Msg::App, Msg::Sparse);
  errstr << "Catalogue subset file name:  " << usnocatname 
           << Sas::message(Msg::App, Msg::Sparse);
  //errstr << "ra=  " << ra << " dec=" << dec << " radius=" << radius 
  //         << Sas::message(Msg::App, Msg::Sparse);

  if (radius<0.0) radius=779.0;

  if (ra < 0.0){ // Get the RA and Dec from the specified file
    //errstr << "L69 Getting RA and DEC from the input image file:  " << inputfilename 
    //	   << Sas::message(Msg::App, Msg::Sparse);
    // Open the file
    DataSet *dataSet = dataSetServer->open(inputfilename, Dal::Read);
    
    if ( !dataSet ) exit(0);
    
    if (dataSet->hasAttribute("RA_PNT")){ 
      
      ra = dataSet->attribute("RA_PNT")->asReal();
      dec = dataSet->attribute("DEC_PNT")->asReal();
    }
    else{
        
      ra = dataSet->attribute("RA_SCX")->asReal();
      dec = dataSet->attribute("DEC_SCX")->asReal();
    }

    dataSetServer->close(dataSet);
    //errstr << "L91 ra=  " << ra << " dec=" << dec  
    //       << Sas::message(Msg::App, Msg::Sparse);

  }

  // Set the environmental variable UB1_PATH
  int stat = putenv("UB1_PATH=http://archive.eso.org/skycat/servers/usnoa-server");
  //    int stat = putenv("UB1_PATH=http://tdc-www.harvard.edu/cgi-bin/scat");

  if (stat != 0)
    {
      cout << "Error stat=" << stat << endl;
      
    }

  //char command[200];
  string catFile = usnocatname;
  sprintf(command, "scat -c ub1 -sm -d -j -n 100000 -r %6.1f  %12.8f %12.8f > %s", 
	  radius, ra, dec, catFile.c_str());
	  //radius, ra, dec, usnocatname);

  errstr << "command:  " << command 
	 << Sas::message(Msg::App, Msg::Sparse);

  system(command);

  vector<double> raList, decList;
  vector<float> errorList, bmagList, rmagList;
  loadData(catFile, raList, decList, errorList, bmagList, rmagList  );

  if ( !createCatalogue( catFile, raList, decList, errorList, bmagList, rmagList ) )
    {
      errstr << "problem with creating the USNO catalogue file  " 
	     << Sas::message(Msg::App, Msg::Sparse);
    }


}


/****************************************
* loadData                              *
*****************************************/
void loadData( string& fileName, vector<double>& ra, vector<double>& dec, 
               vector<float>& error, vector<float>& bmag,  vector<float>& rmag  )
{

    ifstream input( fileName.c_str() );

    const int length = 100;

    if (input.bad())
    {
      
      exit(0);
    }
    
 
   
    while ( input  )
    {


      char line[length];    


      input.getline(line, length);
 
    

      char *text = strtok(line, " ");
                
      text = strtok(NULL, ":");
      if (text == NULL) break;




      char *endptr;



      int r1 = strtol( text, &endptr, 10);

    
 

      text = strtok(NULL, ":");
      if (text == NULL) break;

      int r2 = strtol( text, &endptr, 10);


      text = strtok(NULL, " ");
      if (text == NULL) break;

      double r3 = strtod( text, &endptr);

 
      // Get the declination

      text = strtok(NULL, ":");
      if (text == NULL) break;

      char sign = text[0];
      text[0] = ' ';
       
      int d1 = strtol( text, &endptr, 10);
    
      text = strtok(NULL, ":");
      if (text == NULL) break;

      int d2 = strtol( text, &endptr, 10);

      text = strtok(NULL, " ");
      if (text == NULL) break;

      float d3 = strtod( text, &endptr);

      text = strtok(NULL, " ");
      if (text == NULL) break;
      float mag1 = strtod( text, &endptr);


      text = strtok(NULL, " ");
      if (text == NULL) break;
      float mag2 = strtod( text, &endptr);

      double ra1 = (double(r1) + (double(r2) + r3 / 60.0) / 60.0) * 15.0;



      double dec1 = double(d1) + (double(d2) + d3 / 60.0) / 60.0;
      if (sign == '-') dec1 = -dec1;


      if (mag1 < 1.0) 
      {

	//cout << "Possible bug\n";
	errstr << "Possible bug (negative magnitude)  " 
	       << Sas::message(Msg::App, Msg::Sparse);
	  
      }      
 
      ra.push_back( ra1 );
      dec.push_back( dec1 );
      error.push_back(1.0); 
      bmag.push_back ( mag1 );
      rmag.push_back ( mag2 );


    }


    input.close();





}


/**********************************************************************
*
*********************************************************************/
bool createCatalogue(const string&  catalogueFile, 
                     const vector<double>& ra, 
                     const vector<double>& dec,
                     const vector<float>& radecerr,
                     const vector<float>& bmag,
                     const vector<float>& rmag )

{
                     

    // Open the file
    DataSet *dataSet = dataSetServer->open(catalogueFile, Dal::Create);
  
    if ( !dataSet ) return false;
 
 

    // Add the table
    Table *table = dataSet->addTable( "RAWRES", ra.size() );


    errstr << "catalogue subset size:  " << ra.size() 
           << Sas::message(Msg::App, Msg::Sparse);



    if ( !addColumn( table, "CAT_RA", "degrees", "Right-ascension",   
			   (int)ra.size(), ra) )
      return false;

   


    if ( !addColumn( table, "CAT_DEC", "degrees", "Declination",   
			   (int)dec.size(), dec) )
      return false;


    if ( !addColumn( table, "CAT_RADEC_ERR", "arcsecs", "",   
			   (int)dec.size(), radecerr) )
      return false;




    if ( !addColumn( table, "BMAG", "mag", "",   
			   (int)dec.size(), bmag) )
      return false;


   if ( !addColumn( table, "RMAG", "mag", "",   
			   (int)dec.size(), rmag) )
      return false;




    dataSetServer->close(dataSet);


    return true;
}

/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real32> data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real32, comment, units , size);
 
    ColumnData *columnData = column->data();

    
    real32 * r64 = columnData->real32Data();
    for(int i = 0; i < nData; i++)
        r64[i] = data[i];




    delete columnData;


          
    return true;
};

/********************************************************
*
*
******************************************************/
bool addColumn(Table *table, 
                     const char* columnName, 
                     const char* units,
                     const char* comment,   
		     int nData,
                     vector<real64> data)
{
     

  
    // Get the size of the table
    vector<unsigned long> size(1);
    size[0] = 1;
    size[1] = nData;
  
   
    // Create the column
    Column *column = table->addColumn( columnName, Dal::Real64, comment, units , size);
 
    ColumnData *columnData = column->data();

 
    real64 * r64 = columnData->real64Data();
    for(int i = 0; i < nData; i++)
        r64[i] = data[i];


    delete columnData;

          
    return true;
};


